/* -----------------------------------------------------------------------
 *
 * Project:     ttsnmp
 *
 * Filename:    ttAgentList.c
 *
 * Date:        March 1995
 *
 * Purpose:     Agent list implementation.
 *
 * Author:      Bram van der Waaij      
 *
 *           _o______=====____H_=======_H______=====__        __=====_____H_===
 *          |_______|[[[[[     \#######/       [[[[[ o|      |o ]]]]]      \###
 *     ______| [][] |____    B.D. van der Waaij      ||      || 
 *    |o ____|      |__| \___________________________||      ||________________
 *  |_| /DCC |_7189_|   \_|  |  |  |  |  |  |  |  |  ||_|  |_||  |  |  |  |  | 
 * @|_|=|====|======|=o=|=============================|_|@@|_|=|====|======|===
 *  |_| [(o)==(o)]  |__________________|   [(o)==(o)] |_|\/|_| [(o)==(o)]  |___
 * ============================================================================
 ***[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]*/


#include <UT-SNMP.h>
#include <ttAgentList.h>


typedef struct TAgentListElem {
  char                 agentName[20];   /* name of the agent proc */
  TAgentInfo           info;            /* agent info struct */
  struct TAgentListElem *next;           /* pointer to next agent in list*/
} TAgentListElem;


static TAgentListElem *rootAgentList; /* root pointer to Agent list */


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	CreateAgentList
 *
 * Arguments:	None
 *
 * Return:	OK
 *
 * Purpose:     Reset AgentList root pointer;
 *
 * Remarks:	AgentList must be empty
 *
 * -----------------------------------------------------------------------
 */
STATUS CreateAgentList()
{
  rootAgentList = NULL;

  return OK;
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	RemoveAgentList
 *
 * Arguments:	None
 *
 * Return:	OK
 *
 * Purpose:     Remove all Agents from AgentList
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
STATUS RemoveAgentList()
{
  TAgentListElem *check, *p;

  check = rootAgentList;
  p     = rootAgentList;

  while (check) {     
    check = p->next;
    free(p);
    p = check;
  }  

  rootAgentList = NULL;
  return OK;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AddAgent
 *
 * Arguments:	agentname, the name of the agent (IN)
 *		AgentInfo, the agent information (IN)   
 *
 * Return:	OK or FAILED if no memory
 *
 * Purpose:     Add an agent to the AgentList
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
STATUS AddAgent( AsnOcts *agentname, TAgentInfo AgentInfo )
{
  TAgentListElem *newElem, *p;

    /* create new element */
  newElem = (struct TAgentListElem *) malloc( sizeof( struct TAgentListElem ) );    
  if (newElem == NULL) 
    return FAILED;

  strcpy(newElem->agentName, agentname->octs);
  newElem->info.agentaddr = AgentInfo.agentaddr; 
  newElem->info.retries   = AgentInfo.retries;
  newElem->info.timeout   = AgentInfo.timeout;
  newElem->info.maxrept   = AgentInfo.maxrept;
  newElem->info.nonrptr   = AgentInfo.nonrptr;

  if (!rootAgentList)   /* first element in list */
  {
    newElem->next = NULL;
    rootAgentList = newElem;
    return OK;    
  }

    /* search for last item */
  p = rootAgentList;
  while (p->next) {
    p = p->next;
  }

    /* add newElem to list */
  newElem->next = NULL;
  p->next = newElem;
  return OK;  
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	DelAgent
 *
 * Arguments:	agentname, name of the agent (IN)
 *
 * Return:	OK      if agentname is in the AgentList
 *		FAILED  if agentname is not in the AgentList
 *
 * Purpose:     Remove an agent fomr the AgentList
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
STATUS DelAgent( AsnOcts *agentname )
{
  TAgentListElem *last, *p;

  p    = rootAgentList;
  last = NULL;

  while (p) {
    if (!strcmp( agentname->octs, p->agentName)) {   /* found ? */

      if (last == NULL) {             /* the first in the list */
        rootAgentList = p->next;      /* new root pointer */  
        free(p);
      }         
      else {                          /* somewhere in the list */
        last->next = p->next;         /* skip this Element */
        free(p);
      }

      return OK;
    }
    last = p;
    p = p->next;
  }

  return FAILED;  /* agentname not found */
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	GetAgent
 *
 * Arguments:	agentname, name of the agent (IN)
 *		AgentInfo  the agent information (OUT)
 *
 * Return:	OK     if agentname is found
 *		FAILED if agentname is not found
 *
 * Purpose:     retrieve agent information from agentname
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
STATUS GetAgent( AsnOcts *agentname,  TAgentInfo *AgentInfo )
{
  TAgentListElem *p;

  p = rootAgentList;

  while (p) {
    if (!strcmp( agentname->octs, p->agentName)) {   /* found ? */
      *AgentInfo = p->info;
      return OK;
    }
    p = p->next;
  }

  return FAILED;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	SetAgent
 *
 * Arguments:	agentname, name of the agent (IN)
 *		AgentInfo  the agent information (IN)
 *
 * Return:	OK     if agentname is updated
 *		FAILED if agentname is not found
 *
 * Purpose:     set agent information from agentname
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
STATUS SetAgent( AsnOcts *agentname,  TAgentInfo AgentInfo )
{
  TAgentListElem *p;

  p = rootAgentList;

  while (p) {
    if (!strcmp( agentname->octs, p->agentName)) {   /* found ? */
      p->info = AgentInfo;
      return OK;
    }
    p = p->next;
  }

  return FAILED;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	PrintAgentList
 *
 * Arguments:	none
 *
 * Return:	OK
 *
 * Purpose:     print the AgentList
 *
 * Remarks:	only for test purposes
 *
 * -----------------------------------------------------------------------
 */
STATUS PrintAgentList()
{
  TAgentListElem *p;
  AsnOcts *ipaddr;

  p = rootAgentList;

  if (p == NULL)
    fprintf(stdout,"no agents known\n");
  else
    while (p) {
      GetAddr(&ipaddr, p->info.agentaddr);
      fprintf(stdout,"%s at %s\n", p->agentName, ipaddr->octs);
      p = p->next;
    }

  return OK;
}
