/* ---------------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:	foid.c
 *
 * Date:	September 1994
 * 
 * Purpose:	Converter for Friendly-named Object Identifiers.
 *
 *			The program accepts a dotted OID as argument, both 
 *
 * 			names and numbers, e.g.:
 *
 *					1.3.dod.2
 *
 *			It will translate this, provided that the database
 *
 *			is consistent, into both a friendly name and an 
 * 
 *			ordinary OID.
 *
 * Auther(s):	Erwin Schrijver
 *
 * ---------------------------------------------------------------------------
 */
#include <UT-SNMP.h>
#include <ut-foid.h>

typedef struct ObjectT	{	
		char szObject[O_NAMELEN];
		int	nObject;
		int	nObjDesc;
		struct ObjectT *pNext;
} ObjectT;

/* ---------------------------------------------------------------------------
 *
 * Function:	StringToNum
 *
 * Arguments:	szString, pointer to string, possibly containing 
 *			a number (IN/OUT).
 *
 * Return:	int, number that szString contained ( on success )
 *		and 0 otherwise.
 *
 * Purpose:	This function returns the value of the number in szString, 
 *		if any. If so, szString points to a zero string. Otherwise
 *		the return value is zero.
 *
 * Remarks:	As soon as the routine finds a alphabetical character, it
 *		assumes that szString does not contain a number.
 *
 * ---------------------------------------------------------------------------
 */
	int
StringToNum( char *szString )
{
	char	*cLoop;
	int	nNumber;
	

	for ( cLoop = szString; strncmp( cLoop, EOL, 1 ) ; cLoop++ )
		if ( isalpha( *cLoop ) ) return ( 0 );
		
	nNumber = atoi( szString );
	strncpy( szString, EOL ,1 );
	return ( nNumber );
} 

/* ---------------------------------------------------------------------------
 *
 * Function:	InitObjects
 *
 * Arguments:	szString, pointer to string containing zero or 
 *			more linefeeds (IN).
 *		paTable,  address of pointer to list of objectT (IN).
 *		nEntries, Number of sub-identifiers in szString (IN).
 *
 * Return:	int, OK on success, ERROR when no memory could be allocated
 *
 * Purpose:	This function initialises the array with Objects with the
 *		data provided by szString.
 *
 * Remarks:
 *
 * ---------------------------------------------------------------------------
 */
	int
InitObjects( char *szString, ObjectT **pList, int nEntries )
{
	int	nLoop;
	ObjectT	*pLoop;
			
	pLoop = (ObjectT *) malloc( sizeof( ObjectT ));
	if (pLoop == NULL){
		fprintf( stderr, "InitObjects: couldn't allocate enough memory\n" );
		return ( ERROR );
	}
	strncpy( pLoop->szObject, ROOT, O_NAMELEN );
	pLoop->nObject 	= -1;
	pLoop->nObjDesc = -1;
	pLoop->pNext 	= NULL;
	(*pList) 	= pLoop;
	
	for( nLoop = 0 ; nLoop < nEntries-1 ; nLoop++ ) {
		pLoop->pNext = (ObjectT *) malloc( sizeof( ObjectT ));
		if (pLoop->pNext == NULL){
			fprintf( stderr, "InitObjects: couldn't allocate enough memory\n" );
			return ( ERROR );
		}
		pLoop = pLoop->pNext;
		strncpy( pLoop->szObject, szString, O_NAMELEN );
		if ( strlen( szString ) >= O_NAMELEN )
			strncpy( pLoop->szObject + O_NAMELEN - 1,
				 EOL, 1 );
		szString += (strlen( pLoop->szObject) + 1);
		pLoop->nObject 	= StringToNum( pLoop->szObject );
		pLoop->nObjDesc = -1;
		pLoop->pNext 	= NULL;
	}
	return ( OK );
}
 
/* ---------------------------------------------------------------------------
 *
 * Function:	DotsToEOL
 *
 * Arguments:	szString, pointer to string containing zero or 
 *			more dots (IN/OUT)
 *
 * Return:	int, number of dots on success and FAILED when szString
 *		contains an illegal OID.
 *
 * Purpose:	This function counts the number of dots in a string and 
 *		replaces them with '\0' ( EOL ).
 *
 * Remarks:	Illegal OID's are those beginning or ending with a dot, or 
 *			those that have two successive dots.
 *
 * ---------------------------------------------------------------------------
 */
	int
DotsToEOL( char *szString )
{
	int	nLoop;
	
	/* First character is not allowed to be DOT */
	if ( !strncmp( szString, DOT, 1 )) return FAILED;
	
	for ( nLoop = 0 ; strncmp( szString, EOL, 1) ; szString++ ){
		if ( !strncmp( szString, DOTDOT, 2 )) return FAILED;
		if ( !strncmp( szString, DOT, 1 )) {
			nLoop++;
			strncpy( szString, EOL, 1 );
			continue;
		}
		if ( !isascii( *szString )) return ( FAILED );
	}
	
	/* Last character is not allowed to be DOT */
	if ( !strncmp( --szString, DOT, 1 )) return ( FAILED );
	
	return ( nLoop );
};

/* ---------------------------------------------------------------------------
 *
 * Function:	PrintFriendly
 *
 * Arguments:	pList,  	pointer to list with objects (IN).
 *		octFriendly,	buffer to contain friendly OID (OUT).
 *
 * Return:	int, OK on success and FAILED when octFriendly is too small.
 *
 * Purpose:	This function gives the OID in friendly-name mode.
 *
 * Remarks:
 *
 * ---------------------------------------------------------------------------
 */
 	int
PrintFriendly( ObjectT *pList, AsnOcts octFriendly )
{
	char	szBuf[O_NAMELEN+1];
	
	memset( octFriendly.octs, 0, --octFriendly.octetLen );
	
	while( (pList = pList->pNext) != NULL) {
		if (pList->pNext == NULL){
			if (strlen( pList->szObject ) != 0)
				strncat( octFriendly.octs, pList->szObject,
			 		 octFriendly.octetLen );
			else {
				sprintf( szBuf, "%d", pList->nObject );
				strncat( octFriendly.octs, szBuf,
					 octFriendly.octetLen ); 
			} 	
			if (octFriendly.octetLen <= strlen( pList->szObject ))
				return ( FAILED );
		}		
		else {
			if (strlen( pList->szObject ) != 0)
				sprintf( szBuf, "%s.", pList->szObject );
			else
				sprintf( szBuf, "%d.", pList->nObject );
			strncat( octFriendly.octs, szBuf, octFriendly.octetLen );
			if (octFriendly.octetLen <= strlen( szBuf ))
				return ( FAILED );		
			octFriendly.octetLen -= strlen( szBuf );
		}
	}
	return ( OK );
}

 
/* ---------------------------------------------------------------------------
 *
 * Function:	PrintOID
 *
 * Arguments:	pList,    pointer to list with objects (IN).
 *		octOID,	  buffer to contain normal OID (OUT).
 *
 * Return:	int, OK on success and FAILED when octOID is too small.
 *
 * Purpose:	This function gives the OID in normal mode.
 *
 * Remarks:
 *
 * ---------------------------------------------------------------------------
 */
 	int
PrintOID( ObjectT *pList, AsnOcts octOID )
{
	char	szBuf[O_NAMELEN];
		
	memset( octOID.octs, 0, --octOID.octetLen );

	while( (pList = pList->pNext) != NULL) {	
		if (pList->pNext == NULL){
			sprintf( szBuf, "%i", pList->nObject );
			strncat( octOID.octs, szBuf, octOID.octetLen );
			if (octOID.octetLen <= strlen( szBuf ))
				return ( FAILED );
		}
		else {
			sprintf( szBuf, "%i.", pList->nObject );
			strncat( octOID.octs, szBuf, octOID.octetLen );
			if (octOID.octetLen <= strlen( szBuf ))
				return ( FAILED );		
			octOID.octetLen -= strlen( szBuf );
		}
	}
	return ( OK );
}

/* ---------------------------------------------------------------------------
 *
 * Function:	GetDescIndex
 *
 * Arguments:	*pList,		pointer to list with objects (IN).
 *		*nIndex,	address of description-index (OUT).
 *
 * Return:	int, OK on success and FAILED when no description 
 *			is available.
 *
 * Purpose:	This function gives the index of the description of the
 *		last object.
 *
 * Remarks:
 *
 * ---------------------------------------------------------------------------
 */
	int
GetDescIndex( ObjectT * pList, int *nIndex )
{
	while( pList != NULL ) {
		if (pList->pNext == NULL)
			*nIndex = pList->nObjDesc;
		pList = pList->pNext;
	}
	return ( ((*nIndex) <= 0) ? FAILED : OK );
}

/* ---------------------------------------------------------------------------
 *
 * Function:	FreeObjectList
 *
 * Arguments:	**pList,	address of pointer to list with objects (IN).
 *
 * Return:	int, OK on success and FAILED when pList is NULL. 
 *
 * Purpose:	This function releases the memory used by pList.
 *
 * Remarks:
 *
 * ---------------------------------------------------------------------------
 */
	int
FreeObjectList( ObjectT **pList )
{
	ObjectT	*pTemp;
	
	if (*pList == NULL) return ( FAILED );
	pTemp = *pList;
	while( (*pList = (*pList)->pNext) != NULL ) {
		free( pTemp );
		pTemp = *pList;
	}
	free( pTemp );
	*pList = NULL;
	return ( OK );
}

	 
/* ---------------------------------------------------------------------------
 *
 * Function:	SearchDataBase
 *
 * Arguments:	pData,		pointer to database (array of FoidT) (IN).
 *		pList,		pointer to list with objects (IN/OUT).
 *
 * Return:	int, OK on success and FAILED when OID could not be found.
 *
 * Purpose:	This function searches the database to fill pList.
 *
 * Remarks:
 *
 * ---------------------------------------------------------------------------
 */
	int
SearchDataBase( FoidT *pData, ObjectT *pList )
{
	int	nLoop;
	int	nLineCount=0;	
	
	while( pList->pNext != NULL ) {
		if ( !strncmp( pList->pNext->szObject, EOL, 1 )){
			if( ( SearchBranchName( pData, pList, &nLineCount )) 
						== FAILED )
				return FAILED;
		}
		else
		{
			if( ( SearchBranchNumber( pData, &pList, &nLineCount ))
						== FAILED )
				return FAILED;
		}
		pList = pList->pNext;
	}
	return ( OK );
	
}
			
/* ---------------------------------------------------------------------------
 *
 * Function:	SearchBranchName
 *
 * Arguments:	pData,		pointer to database (IN).
 *		pList,		pointer to object-list (IN/OUT).
 *		pLine,		pointer to current place in database (IN/OUT).
 *
 * Return:	int, OK on success and FAILED when the desired branch 
 *		could not be found.	
 *
 * Purpose:	This function searches the database for the given branch-number
 *
 * Remarks:
 *
 * ---------------------------------------------------------------------------
 */
	int
SearchBranchName( FoidT *pData, ObjectT *pList, int *pLine )
{
	for (;(pData[*pLine].nBranch != -1) || (pData[*pLine].nDesIndex != -1);
								 (*pLine)++ )		
		if ( !strcmp( pList->szObject, pData[*pLine].szNode ) && 
		   ( pList->pNext->nObject == pData[*pLine].nBranch ) ){
			strncpy( pList->pNext->szObject,
				 pData[*pLine].szBranch, O_NAMELEN );
			pList->pNext->nObjDesc = pData[*pLine].nDesIndex;
			return 	( OK );
		}
		
	return ( FAILED );
} 
 
/* ---------------------------------------------------------------------------
 *
 * Function:	SearchBranchNumber
 *
 * Arguments:	pData,		pointer to database (IN).
 *		pList		pointer to object-list (IN/OUT).
 *		pLine,		pointer to current place in database (IN/OUT).
 *
 * Return:	int, OK on success and FAILED when the desired branch 
 *		could not be found. 	
 *
 * Purpose:	This function searches the database for the given branch-name.
 *
 * Remarks:
 *
 * ---------------------------------------------------------------------------
 */
	int
SearchBranchNumber( FoidT *pData, ObjectT **pList, int *pLine )
{
	for (;(pData[*pLine].nBranch != -1) || (pData[*pLine].nDesIndex != -1); 
								(*pLine)++ )
		if (!strcmp( (*pList)->pNext->szObject, pData[*pLine].szBranch)){
			(*pList)->pNext->nObject = pData[*pLine].nBranch;
			(*pList)->pNext->nObjDesc = pData[*pLine].nDesIndex;
			if (strcmp( (*pList)->szObject, pData[*pLine].szNode ))
				return (SearchParent( pData, pList, *pLine )); 
			return ( OK );
		}
	
	return ( FAILED );
} 



/* ---------------------------------------------------------------------------
 *
 * Function:	SearchParent
 *
 * Arguments:	pData,		pointer to database (IN).
 *		pList		address of pointer to object-list (IN).
 *		nLine,		current place in database (IN).
 *
 * Return:	int, OK on success and FAILED when the desired node(s) 
 *		could not be found. 	
 *
 * Purpose:	This function searches the database backwards to obtain the
 *		parent-nodes.
 *
 * Remarks:
 *
 * ---------------------------------------------------------------------------
 */
	int
SearchParent( FoidT *pData, ObjectT **pList, int nLine )
{
	char	*szNode;
	int	nInserted = 0;
	
	szNode = pData[nLine].szNode;
	
	for( ; nLine >= 0 ; nLine-- )
		if (!strcmp( szNode, pData[nLine].szBranch )){
			InsertObject( pData[nLine], (*pList) );
			nInserted++;
			szNode = pData[nLine].szNode;
			if (!strcmp( szNode, (*pList)->szObject )){
				for ( ; nInserted > 0 ; 
				        	(*pList) = (*pList)->pNext, 
				        	nInserted-- );
				return ( OK );
			};			
		}
	return ( FAILED );
}	 	

/* ---------------------------------------------------------------------------
 *
 * Function:	InsertObject
 *
 * Arguments:	Data,		Data to fill object (IN).
 *		pList		pointer to object-list (IN).
 *
 * Return:	int, OK on success and ERROR when no memory could 
 *		be allocated
 *
 * Purpose:	This function inserts a object in the object-list and fills
 *		it with the data.
 *
 * Remarks:
 *
 * ---------------------------------------------------------------------------
 */
	int
InsertObject( FoidT Data, ObjectT *pList )
{
	ObjectT	*pTemp;
	
	pTemp = pList->pNext;
	
	pList->pNext = (ObjectT *) malloc( sizeof( ObjectT ));
	if (pList->pNext == NULL){
		fprintf( stderr, "InsertObject: couldn't allocate enough memory.\n" );
		return ( ERROR );
	}
	
	pList = pList->pNext;
	pList->pNext = pTemp;
	
	strcpy( pList->szObject, Data.szBranch );
	pList->nObject = Data.nBranch;
	pList->nObjDesc = Data.nDesIndex;
	
	return ( OK );
}
	
/* ---------------------------------------------------------------------------
 *
 * Function:	FillFoidBase
 *
 * Arguments:	pfFile,		pointer to file-stream (IN).
 *		pData,		pointer to database (IN).
 *		nLength,	number of lines to read (IN).
 *
 * Return:	int, OK on success, FAILED when datafile is illegal.	
 *
 * Purpose:	This function reads the database to fill the database in memory.
 *
 * Remarks:
 *
 * ---------------------------------------------------------------------------
 */
	int
FillFoidBase( FILE *pfFile, FoidT* pData, int nLength )
{
	int 	nLoop;
	char	szLine[LINELEN];
	char	szBufBranch[LINELEN];
	char	szBufNode[LINELEN];
			
	for ( nLoop = 0 ; nLoop < nLength ; nLoop++ ){
		while (1){
			if ( fgets( szLine, LINELEN, pfFile ) == NULL )
				return ( FAILED );
	
		      	if (strncmp( szLine, COMMENT, strlen(COMMENT) ))
		      		break;
		}
		
		strncpy( pData[nLoop].szBranch, UNKNOWN, O_NAMELEN );
		strncpy( pData[nLoop].szNode,   UNKNOWN, O_NAMELEN );
		pData[nLoop].nBranch   = -1;
		pData[nLoop].nDesIndex = -1;
		
		if ( sscanf( szLine, "%s\t%s\t%d\t%d",
					szBufBranch, szBufNode,	
					&pData[nLoop].nBranch,
					&pData[nLoop].nDesIndex ) < 3 )
			return ( FAILED );
		
		strncpy( pData[nLoop].szBranch, szBufBranch, O_NAMELEN );
		if ( strlen( szBufBranch ) >= O_NAMELEN )
			strncpy( pData[nLoop].szBranch + O_NAMELEN - 1,
				 EOL, 1 );
		strncpy( pData[nLoop].szNode,   szBufNode,   O_NAMELEN );
		if ( strlen( szBufNode ) >= O_NAMELEN )
			strncpy( pData[nLoop].szNode + O_NAMELEN - 1,
				 EOL, 1 );
	}
	return ( OK );
}

/* ---------------------------------------------------------------------------
 *
 * Function:	foidClose
 *
 * Arguments:	pData,		address of pointer to database (IN/OUT).
 *
 * Return:	int, OK on success, FAILED when database was already closed.	
 *
 * Purpose:	This function releases memory.
 *
 * Remarks:
 *
 * ---------------------------------------------------------------------------
 */
	int
foidClose( FoidT **pData )
{
	if (*pData == NULL)
		return ( FAILED );
	free( *pData );
	*pData == NULL;
	return ( OK );
}

/* ---------------------------------------------------------------------------
 *
 * Function:	foidOpen
 *
 * Arguments:	pData,	address of pointer to FoidT to 
 *			contain database (IN/OUT).
 *
 * Return:	OK on success, ERROR when datafile couldn't be opened,
 *		or when datafile is incorrect, or when not enough memory
 *		could be allocated. 	
 *
 * Purpose:	This function reads the entire database in memory
 *
 * Remarks:	This function should be called before any convertion is made.
 *		After all convertions, before the end of the program, the
 *		used memory has to be freed by calling foidClose().
 *
 * ---------------------------------------------------------------------------
 */
	int
foidOpen( FoidT **pData )
{
	FILE	*pfFile;
	char	szLine[LINELEN];
	char	szDataBase[ARG_LEN];
	char	*szPath;
	int	nLength=0;	
	
	if( (szPath = (char *) getenv( DATA_PATH )) == NULL ){
		fprintf( stderr, "foidOpen: undeclared environment SNMP_PARTYMIB_INITPATH\n" );
		return ( ERROR );
	}
	
	strncpy( szDataBase, szPath, ARG_LEN );
	strncat( szDataBase, TREE, ARG_LEN - strlen( szPath ) - 1 );
	
	if( (pfFile = fopen( szDataBase, "r" )) == NULL ){
		fprintf( stderr, "foidOpen: unable to open database foid.tree.\n" );
		return ( ERROR );
	}
		
	while ( fgets( szLine, LINELEN, pfFile ) != NULL  )
		if ( strncmp( szLine, COMMENT, strlen(COMMENT) ))
			nLength++;
			
	rewind( pfFile );
	
	if (nLength == 0){
		fclose( pfFile );
		fprintf( stderr, "foidOpen: incorrect datafile foid.tree\n" );
		return ( ERROR );
	}
	
	(*pData) = (FoidT*) calloc( (nLength + 1), sizeof( FoidT ) );
	if (*pData == NULL){
		fprintf( stderr, "foidOpen: couldn't allocate enough memory.\n" );
		fclose( pfFile );
		return ( ERROR );
	}
	
	if ( FillFoidBase( pfFile, *pData, nLength ) == FAILED ){
		fprintf( stderr, "foidOpen: incorrect datafile foid.tree.\n" );
		free( *pData );
		fclose( pfFile );
		return ( ERROR );
	}
	
	strncpy( (*pData)[nLength].szBranch, EOL, 1 );
	strncpy( (*pData)[nLength].szNode, EOL, 1 );
	(*pData)[nLength].nBranch   = -1;
	(*pData)[nLength].nDesIndex = -1;
	 
	fclose( pfFile );
	return ( OK );
}

/* ---------------------------------------------------------------------------
 *
 * Function:	foidConv
 *
 * Arguments:	pData,		pointer to database (IN).
 *		szIn,		string with mixed OID (IN).
 *		octFriendly,	string with will be filled with friendly name (OUT).
 *		octOID,		string with will be filled with OID (OUT).
 *		nDesIndex,	address of Indexnumber to description. This
 *				address will contain zero if no convertion is 
 *				made ( due to an error or fault ), and -1 when
 *				the object has no description (OUT).
 *		nError,		Status of the conversion (OUT):
 *			ILLEGAL when OID is illegal,
 *			NBUF	when buffer(s) is/are too small,
 *			NTOTAL	when database isn't sufficient.
 *
 * Return:	int, OK on success, FAILED when OID isn't correct or when
 *		OID could not be found, or when octOID/octFriendly is/are
 *		too small.
 *		ERROR when not enough memory could be allocated.	
 *
 * Purpose:	This function converts a mixed OID.
 *
 * Remarks:
 *
 * ---------------------------------------------------------------------------
 */
	int
foidConv( FoidT *pData, char *szIn, AsnOcts octFriendly, 
	  AsnOcts octOID, int *nDesIndex, int *nError )
{
	ObjectT	*paObjects;
	int	nObjects;
	int	nResult;
	char	szInput[ARG_LEN];
	
	strncpy( szInput, szIn, ARG_LEN );
	if ( strlen( szIn ) >= ARG_LEN )
		strncpy( szInput + ARG_LEN - 1, EOL, 1 );
	strncpy( octFriendly.octs, EOL, 1);
	strncpy( octOID.octs, EOL, 1);
	*nDesIndex = 0;
	*nError = 0;
	
	if( (nObjects = DotsToEOL( szInput )) == FAILED){
		*nError = ILLEGAL;
		return ( FAILED );
	}
	nObjects += 2;
	
	InitObjects( szInput, &paObjects, nObjects );
			
	if (SearchDataBase( pData, paObjects ) == FAILED)
		*nError = NTOTAL;	
	  
	if (PrintFriendly( paObjects, octFriendly ) == FAILED)
		*nError = NBUF;	
	if (PrintOID( paObjects, octOID ) == FAILED)
		*nError = NBUF;
			
	GetDescIndex( paObjects, nDesIndex );
		
	FreeObjectList( &paObjects );
	
	if (*nError != 0) return ( FAILED );
	return ( OK );	
} 
 

/* ---------------------------------------------------------------------------
 *
 * Function:	ReadDescription
 *
 * Arguments:	pfData,		pointer to stream of database-file (IN).
 *		octLine,	Buffer that contains first line (IN/OUT).
 *		octBuf,		Buffer to contain description (OUT).
 *
 * Return:	int, OK on succes, ERROR when database is incorrect,
 *		FAILED when buffer is too small.
 *
 * Purpose:	This function reads the description that is located
 *		immediately beyond the current file-pointer.
 *
 * Remarks:
 *
 * ---------------------------------------------------------------------------
 */
	int
ReadDescription( FILE *pfData, AsnOcts octLine, AsnOcts octBuf )
{
	int	nLoop;
	char	cSep;
	
	strncpy( octBuf.octs + (--octBuf.octetLen), EOL, 1 );
	
	while ( fgets( octLine.octs, octLine.octetLen, pfData ) != NULL ){
		strncat( octBuf.octs, octLine.octs, 
			 octBuf.octetLen - strlen( octBuf.octs ) );
		
		if( (octBuf.octetLen - strlen( octBuf.octs )) <= 0 )
			return ( FAILED );
		
		for( nLoop = strlen( octLine.octs ) ; nLoop >= 0 ; nLoop-- ){
			strncpy( &cSep, octLine.octs+nLoop, 1 );
			if ( cSep == '\"' ) return ( OK );
			if ( isalpha( cSep )) break;
		}
	}
	fprintf( stderr, "ReadDescription: datafile foid.desc is incorrect\n" );
	return ( ERROR ); 
}

/* ---------------------------------------------------------------------------
 *
 * Function:	foidDescription
 *
 * Arguments:	nIndex,		index of description (IN).
 *		octBuf,		Buffer to contain description (OUT).
 *
 * Return:	int, OK on succes, ERROR when database couldn't be opened
 *		or when database is incorrect.
 *		FAILED when desciption isn`t in database, or when octBuf is
 *		too small. 	
 *
 * Purpose:	This function gives the discription refered to by nIndex.
 *
 * Remarks:
 *
 * ---------------------------------------------------------------------------
 */
	int
foidDescription( int nIndex, AsnOcts octBuf )
{
	FILE	*pfData;
	char	*szPath;
	char	szDataBase[ARG_LEN];
	AsnOcts	octLine;
	int	nLine;
	int	nResult;
	
	strncpy( octBuf.octs, EOL, 1 );

	if (nIndex <= 0) return ( FAILED );
	
	if( (szPath = (char *) getenv( DATA_PATH )) == NULL ){
		fprintf( stderr, "foidDescription: undeclared environment SNMP_PARTYMIB_INITPATH.\n" );
		return ( ERROR );
	}
	
	strncpy( szDataBase, szPath, ARG_LEN );
	strncat( szDataBase, DESC, ARG_LEN - strlen( szPath ) );
	
	if( (pfData = fopen( szDataBase, "r" )) == NULL ){
		fprintf( stderr, "foidDescription: unable to open database foid.desc.\n" );
		return ( ERROR );
	}
		
	octLine.octs = (unsigned char *) malloc( LINELEN );
	octLine.octetLen = LINELEN;
	strncpy( octLine.octs, EOL, 1 );
	strncpy( octLine.octs + (--octLine.octetLen), EOL, 1 );
	
	while( fgets( octLine.octs, octLine.octetLen, pfData ) != NULL ){
		if (!strncmp( octLine.octs, COMMENT, strlen(COMMENT) ))
			continue;	
		if( (sscanf( octLine.octs, "%d\n", &nLine )) == 1 ){
			if (nLine != nIndex) continue;
			nResult = ReadDescription( pfData, octLine, octBuf );
			free( octLine.octs );
			fclose( pfData );
			return ( nResult );	
		}	
	}
	
	free( octLine.octs );
	fclose( pfData );
	return ( FAILED );
}  


/* ---------------------------------------------------------------------------
 */
